'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _arguments = arguments;
var getDisplayName = exports.getDisplayName = function getDisplayName(component) {
  return component.displayName || component.name || 'Component';
};

// Returns a function, that, as long as it continues to be invoked, will not
// be triggered. The function will be called after it stops being called for
// N milliseconds. If `immediate` is passed, trigger the function on the
// leading edge, instead of the trailing.
var debounce = exports.debounce = function debounce(func, wait, immediate) {
  var timeout = void 0;
  return function () {
    var context = undefined;
    var args = _arguments;
    var later = function later() {
      timeout = null;
      if (!immediate) func.apply(context, args);
    };
    var callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
    if (callNow) func.apply(context, args);
  };
};

var noop = /* istanbul ignore next */exports.noop = function noop() {};

var fakeChangeEvent = exports.fakeChangeEvent = function fakeChangeEvent(name, value) {
  return {
    preventDefault: noop,
    target: { name: name, value: value }
  };
};

var shallowCompare = exports.shallowCompare = function shallowCompare(nextObj, currObj) {
  var exclude = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var keysNext = Object.keys(nextObj).filter(function (v) {
    return !exclude.includes(v);
  });
  var keysCurr = Object.keys(currObj).filter(function (v) {
    return !exclude.includes(v);
  });
  return keysNext.length !== keysCurr.length || keysNext.some(function (next) {
    return nextObj[next] !== currObj[next];
  });
};
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(getDisplayName, 'getDisplayName', 'src/utilities.js');

  __REACT_HOT_LOADER__.register(debounce, 'debounce', 'src/utilities.js');

  __REACT_HOT_LOADER__.register(noop, 'noop', 'src/utilities.js');

  __REACT_HOT_LOADER__.register(fakeChangeEvent, 'fakeChangeEvent', 'src/utilities.js');

  __REACT_HOT_LOADER__.register(shallowCompare, 'shallowCompare', 'src/utilities.js');
}();

;